/***********************************************************************************************************************
  GCC-spezifier Code für den Bootloader
------------------------------------------------------------------------------------------------------------------------
  \file       bootloader_util_gcc.c
  \creation   2014-02-21, Joe Merten
***********************************************************************************************************************/

#include "bootloader_util.h"
#include <stdint.h>
#include "nordic_common.h"
#include "bootloader_types.h"
#include "dfu_types.h"

uint8_t __attribute__((section (".bootloader_settings_sect"))) m_boot_settings[CODE_PAGE_SIZE] __attribute__((used));          /**< This variable reserves a codepage for bootloader specific settings, to ensure the compiler doesn't locate any code or variables at his location. */
uint32_t __attribute__((section (".NRF_UICR_BOOT_START_SECT"))) m_uicr_bootloader_start_address = BOOTLOADER_REGION_START;      /**< This variable ensures that the linker script will write the bootloader start address to the UICR register. This value will be written in the HEX file and thus written to UICR when the bootloader is flashed into the chip. */

void bootloader_util_app_start(uint32_t start_addr)
{
#if 0
    { /* First approach to start the application */
        typedef void (*application_main_t)();
        application_main_t application_main = *(application_main_t*)(start_addr+4);
        application_main();
    }
#else
    { /* Second approach to start the application */
        asm volatile(" LDR   R0, =0x14000          \n\t" // Assign app code address
                     " LDR   R2, [R0]              \n\t" // Get App MSP
                     " MSR   MSP, R2               \n\t" // Set the main stack pointer to the applications MSP
                     " LDR   R3, [R0, #0x00000004] \n\t" // Get application reset vector address
                     " BX    R3                    \n\t" // No return - stack code is now activated only through SVC and plain interrupts
                     " .ALIGN                      ");
    }
#endif
}

void bootloader_util_settings_get(const bootloader_settings_t** pp_bootloader_settings)
{
    // Read only pointer to bootloader settings in flash.
    *pp_bootloader_settings = (bootloader_settings_t*)BOOTLOADER_SETTINGS_ADDRESS;
}
